"""kubetest -- a Kubernetes integration test framework in Python."""

__title__ = "kubetest"
__version__ = "0.9.4"
__description__ = "A Kubernetes integration test framework in Python."
__author__ = "Vapor IO"
__author_email__ = "vapor@vapor.io"
__url__ = "https://github.com/vapor-ware/kubetest"
__license__ = "GNU General Public License v3.0"
